package org.fuseri.modulecertificate;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.certificate.CertificateCreateDto;
import org.fuseri.model.dto.certificate.CertificateSimpleDto;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.AddressDto;
import org.fuseri.model.dto.user.UserDto;
import org.fuseri.modulecertificate.service.CertificateController;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.time.Instant;
import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;


@SpringBootTest
@AutoConfigureMockMvc
class CertificateControllerTests {

    private final UserDto USER = new UserDto("novakovat",
            "novakova@gamil.com", "Tereza", "Nováková", new AddressDto());
    private final CourseDto COURSE = new CourseDto(1L, "AJ1", 10,
            LanguageTypeDto.ENGLISH, ProficiencyLevelDto.A1);
    private final CertificateCreateDto certificateCreateDto = new CertificateCreateDto(USER, COURSE);
    private final CertificateSimpleDto certificateDto = new CertificateSimpleDto(0L, USER.getId(),
            Instant.now(), COURSE.getId(), "", "");
    @Autowired
    private MockMvc mockMvc;
    @MockBean
    private CertificateController certificateController;

    private static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    void generateCertificate() throws Exception {
        Mockito.when(certificateController.generate(ArgumentMatchers.any(CertificateCreateDto.class)))
                .thenReturn(certificateDto);

        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(certificateCreateDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id").value(certificateDto.getId()))
                .andExpect(jsonPath("$.userId").value(certificateDto.getUserId()))
                .andExpect(jsonPath("$.generatedAt").value(certificateDto.getGeneratedAt().toString()))
                .andExpect(jsonPath("$.courseId").value(certificateDto.getCourseId()))
                .andExpect(jsonPath("$.certificateFile").value(certificateDto.getCertificateFile()))
                .andExpect(jsonPath("$.certificateFileName").value(certificateDto.getCertificateFileName()));

    }

    @Test
    void generateCertificateWithNullUser() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(null, COURSE)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void generateCertificateWithNullCourse() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(USER, null)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void generateCertificateWithoutParams() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificate() throws Exception {
        Mockito.when(certificateController.find(ArgumentMatchers.anyLong())).thenReturn(certificateDto);

        mockMvc.perform(get("/certificates/find").param("id", certificateDto.getId().toString()))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id").value(certificateDto.getId()));
    }

    @Test
    void findCertificateWithoutId() throws Exception {
        mockMvc.perform(get("/certificates/find"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificatesForUser() throws Exception {
        Mockito.when(certificateController.findForUser(ArgumentMatchers.anyLong())).thenReturn(List.of(certificateDto));

        mockMvc.perform(get("/certificates/findForUser").param("userId", "0"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$").isNotEmpty());
    }

    @Test
    void findCertificatesWithoutUserId() throws Exception {
        mockMvc.perform(get("/certificates/findForUser"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificateIdForUserAndCourse() throws Exception {
        Mockito.when(certificateController.findForUserAndCourse(ArgumentMatchers.anyLong(),
                        ArgumentMatchers.anyLong()))
                .thenReturn(List.of(certificateDto));

        mockMvc.perform(get("/certificates/findForUserAndCourse")
                        .param("userId", "0")
                        .param("courseId", "0"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$").isNotEmpty());
    }

    @Test
    void findCertificateIdWithoutUserId() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse")
                        .param("courseId", "0"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificateIdWithoutCourseId() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse")
                        .param("userId", "0"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificateIdWithoutParams() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void deleteCertificate() throws Exception {
        Mockito.doNothing().when(certificateController).delete(ArgumentMatchers.anyLong());

        mockMvc.perform(delete("/certificates/delete")
                        .param("id", String.valueOf(0L)))
                .andExpect(status().isOk());
    }

    @Test
    void deleteCertificateWithoutParam() throws Exception {
        mockMvc.perform(delete("/certificates/delete"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findAllCertificates() throws Exception {
        Mockito.when(certificateController.findAllCertificates(ArgumentMatchers.any(Pageable.class)))
                .thenReturn(Page.empty(PageRequest.of(0, 1)));

        mockMvc.perform(get("/certificates/findAll")
                        .param("page", "0")
                        .param("size", "10"))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.content").isEmpty());
    }

    @Test
    void findAllCertificatesWithoutParam() throws Exception {
        Mockito.when(certificateController.findAllCertificates(ArgumentMatchers.any(Pageable.class)))
                .thenReturn(Page.empty(PageRequest.of(0, 1)));

        mockMvc.perform(get("/certificates/findAll"))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.content").isEmpty());
    }
}
